@@ -2,15 +2,27 @@ from datetime  import datetime
 from functools import partial as functools_partial
 
 import netCDF4
-if netCDF4.__version__ >= '1.2.5':
+if netCDF4.__version__ >= '1.4':
+    import cftime
+    _netCDF4_netcdftime_parse_date = cftime._parse_date
+elif netCDF4.__version__ >= '1.2.5':
     from netCDF4 import netcdftime
     _netCDF4_netcdftime_parse_date = netcdftime._parse_date
     
-from numpy import around    as numpy_around
-from numpy import array     as numpy_array
-from numpy import ndarray   as numpy_ndarray
-from numpy import ndim      as numpy_ndim
-from numpy import vectorize as numpy_vectorize
+from numpy import around     as numpy_around
+from numpy import array      as numpy_array
+from numpy import asanyarray as numpy_asanyarray
+from numpy import ndarray    as numpy_ndarray
+from numpy import ndim       as numpy_ndim
+from numpy import vectorize  as numpy_vectorize
+
+from numpy.ma import array        as numpy_ma_array
+from numpy.ma import isMA         as numpy_ma_isMA
+from numpy.ma import is_masked    as numpy_ma_is_masked
+from numpy.ma import masked       as numpy_ma_masked
+from numpy.ma import masked_all   as numpy_ma_masked_all
+from numpy.ma import masked_where as numpy_ma_masked_where
+from numpy.ma import nomask       as numpy_ma_nomask
 
 from .functions import inspect as cf_inspect
 from .units     import Units
@@ -32,8 +44,12 @@ _calendar_months = Units('calendar_months')
 # Datetime object
 #
 # ====================================================================
+if netCDF4.__version__ < '1.4':
+    Parent = netCDF4.netcdftime.datetime
+else:
+    Parent = cftime.datetime
 
-class Datetime(netCDF4.netcdftime.datetime):
+class Datetime(Parent):
     '''A date-time object which supports CF calendars.
 
 Any date-time in any CF calendar is allowed.
@@ -71,34 +87,34 @@ Addition and subtraction operations are defined for `cf.Datetime` and
 `cf.TimeDuration` objects:
 
 >>> cf.Datetime(2003, 2, 1) + cf.D(30)
-<CF Datetime: 2003-03-03 00:00:00>
+<CF Datetime: 2003-03-03T00:00:00Z>
 >>> cf.M(30) + cf.Datetime(2003, 2, 1)
-<CF Datetime: 2003-03-03 00:00:00>
+<CF Datetime: 2003-03-03T00:00:00Z>
 >>> cf.Datetime(2003, 2, 1, calendar='360_day') + cf.D(30)
-<CF Datetime: 2003-03-01 00:00:00 360_day>
+<CF Datetime: 2003-03-01T00:00:00Z 360_day>
 >>> cf.Datetime(2003, 2, 1, calendar='360_day') - cf.D(30)
-<CF Datetime: 2003-01-01 00:00:00 360_day>
+<CF Datetime: 2003-01-01T00:00:00Z 360_day>
 
 >>> cf.Datetime(2003, 2, 1) - cf.h(30)
-<CF Datetime: 2003-01-30 18:00:00>
+<CF Datetime: 2003-01-30T18:00:00Z>
 >>> cf.s(456) + cf.Datetime(2003, 2, 1)
-<CF Datetime: 2003-02-01 00:07:36>
+<CF Datetime: 2003-02-01T00:07:36Z>
 
 >>> cf.Datetime(2003, 2, 1) + cf.M(1)
-<CF Datetime: 2003-03-01 00:00:00>
+<CF Datetime: 2003-03-01T00:00:00Z>
 >>> cf.M(1) + cf.Datetime(2003, 2, 1, calendar='360_day') 
-<CF Datetime: 2003-03-01 00:00:00 360_day>
+<CF Datetime: 2003-03-01T00:00:00Z 360_day>
 >>> cf.Datetime(2003, 2, 1, calendar='noleap') + cf.M(245)
-<CF Datetime: 2023-07-01 00:00:00 noleap>
+<CF Datetime: 2023-07-01T00:00:00Z noleap>
 >>> cf.Datetime(2003, 2, 1) - cf.Y(2)
-<CF Datetime: 2001-02-01 00:00:00>
+<CF Datetime: 2001-02-01T00:00:00Z>
 >>> cf.Datetime(2003, 2, 1) - cf.Y(2) + cf.D(10)
-<CF Datetime: 2001-02-11 00:00:00>
+<CF Datetime: 2001-02-11T00:00:00Z>
 
 >>> d = cf.Datetime(2003, 2, 1)
 >>> d += cf.s(34567689)
 >>> d
-<CF Datetime: 2004-03-07 02:08:09>
+<CF Datetime: 2004-03-07T02:08:09Z>
 
 **Attributes**
 
@@ -155,6 +171,7 @@ True
             self.microsecond = 0
 
             #        if netCDF4.__version__ <= '1.2.4':
+
         self._calendar = calendar
     #--- End: def
 
@@ -199,11 +216,9 @@ x__str__() <==> str(x)
 
 '''   
         args = self.timetuple()[:6]
-        out = '{0: >4}-{1:0>2}-{2:0>2} {3:0>2}:{4:0>2}:{5:0>2}Z'.format(*args)
+        out = '{0: >4}-{1:0>2}-{2:0>2}T{3:0>2}:{4:0>2}:{5:0>2}Z'.format(*args)
         if self.microsecond:
-            out = out.replace('Z', '.{0:0>6}'.format(self.microsecond))      
-        else:
-            out = out.replace('Z', '')
+            out = out.replace('Z', '.{0:0>6}Z'.format(self.microsecond))      
         return out
     #--- End: def
 
@@ -296,6 +311,16 @@ x__lt__(y) <==> x<y
             return NotImplemented
     #--- End: def
 
+    @property
+    def calendar(self):
+        '''
+        '''
+        calendar = self._calendar
+        if not calendar:
+            return None
+        
+        return calendar
+    #--- End: def
     def copy(self):
         '''
 
@@ -348,9 +373,7 @@ d.minute, d.second, d.microsecond)``.
     #--- End: def
 
     def inspect(self):
-        '''
-
-Inspect the attributes.
+        '''Inspect the attributes.
 
 .. seealso:: `cf.inspect`
 
@@ -358,7 +381,7 @@ Inspect the attributes.
 
     None
 
-'''
+        '''
         print cf_inspect(self)
     #--- End: def
 
@@ -379,15 +402,15 @@ Inspect the attributes.
     def strftime(self, format=None):
         '''Return a string representing the date-time.
 
-The string is an ISO 8601 string of the form Y-MM-DD hh:mm:ss, for
-example ``2000-01-02 03:04:05``, or Y-MM-DD hh:mm:ss.s where the
+The string is an ISO 8601 string of the form Y-MM-DDThh:mm:ssZ, for
+example ``2000-01-02T03:04:05Z``, or Y-MM-DDThh:mm:ss.sZ where the
 final "s" refers to any sized decimal fraction added to the seconds,
-for example ``2000-01-02 03:04:05.000786``.
+for example ``2000-01-02T03:04:05.000786Z``.
 
 :Examples 1:
 
 >>> cf.Datetime(2004, 1, 30).strftime()
-'2004-01-30 00:00:00'
+'2004-01-30T00:00:00Z'
 
 :Parameters:
 
@@ -402,16 +425,16 @@ for example ``2000-01-02 03:04:05.000786``.
 :Examples 2:
 
 >>> cf.Datetime(2004, 2, 30).strftime()
-'2004-02-30 00:00:00'
+'2004-02-30T00:00:00Z'
 
 >>> cf.Datetime(2004, 2, 30, calendar='360_day').strftime()
-'2004-02-30 00:00:00'
+'2004-02-30T00:00:00Z'
 
 >>> cf.Datetime(2004, 7, 4, 12, 45, 56, 7456).strftime()
-'2004-07-04 12:45:56.007456'
+'2004-07-04T12:45:56.007456Z'
 
 >>> cf.Datetime(2004, 12, 34).strftime()
-'2004-12-34 00:00:00'
+'2004-12-34T00:00:00Z'
 
         '''
         return '{0: >4}-{1:0>2}-{2:0>2} {3:0>2}:{4:0>2}:{5:0>2}'.format(
@@ -427,7 +450,7 @@ Return the current Gregorian calendar UTC date and time.
 :Examples 1:
 
 >>> cf.Datetime.utcnow()
-<CF Datetime: 2016-10-10 12:15:24.002376>
+<CF Datetime: 2016-10-10T12:15:24.002376Z>
 
 :Returns:
 
@@ -437,12 +460,12 @@ Return the current Gregorian calendar UTC date and time.
 :Examples 2:
 
 >>> cf.Datetime.utcnow()
-<CF Datetime: 2016-10-10 12:15:25.422179>
+<CF Datetime: 2016-10-10T12:15:25.422179Z>
 >>> d = cf.Datetime(2005, 6, 7)
 >>> d.utcnow()
-<CF Datetime: 2016-10-10 12:15:50.375309>
+<CF Datetime: 2016-10-10T12:15:50.375309Z>
 >>> d
-<CF Datetime: 2005-06-07 00:00:00>
+<CF Datetime: 2005-06-07T00:00:00Z>
 
 '''
         return cls(*elements(datetime.utcnow()))
@@ -450,6 +473,19 @@ Return the current Gregorian calendar UTC date and time.
 
 #--- End: class
 
+#def _strftime(x):
+#    try:
+#        out = x.strftime('%Y-%m-%d %H:%M:%S')
+#    except ValueError:
+#        out = '{0}-{1:0>2}-{2:0>2} {3:0>2}:{4:0>2}:{5:0>2}'.format(
+#            *self.timetuple()[:6])
+#        
+#    if self.microsecond:
+#        out = out.replace('Z', '.{0:0>6}Z'.format(self.microsecond))
+#        
+#    return out
+
+
 #class Datetime2(object):
 #
 #    # ----------------------------------------------------------------
@@ -491,34 +527,34 @@ Return the current Gregorian calendar UTC date and time.
 #`cf.TimeDuration` objects:
 #
 #>>> cf.Datetime(2003, 2, 1) + cf.D(30)
-#<CF Datetime: 2003-03-03 00:00:00>
+#<CF Datetime: 2003-03-03T00:00:00Z>
 #>>> cf.M(30) + cf.Datetime(2003, 2, 1)
-#<CF Datetime: 2003-03-03 00:00:00>
+#<CF Datetime: 2003-03-03T00:00:00Z>
 #>>> cf.Datetime(2003, 2, 1, calendar='360_day') + cf.D(30)
-#<CF Datetime: 2003-03-01 00:00:00 360_day>
+#<CF Datetime: 2003-03-01T00:00:00Z 360_day>
 #>>> cf.Datetime(2003, 2, 1, calendar='360_day') - cf.D(30)
-#<CF Datetime: 2003-01-01 00:00:00 360_day>
+#<CF Datetime: 2003-01-01T00:00:00Z 360_day>
 #
 #>>> cf.Datetime(2003, 2, 1) - cf.h(30)
-#<CF Datetime: 2003-01-30 18:00:00>
+#<CF Datetime: 2003-01-30T18:00:00Z>
 #>>> cf.s(456) + cf.Datetime(2003, 2, 1)
-#<CF Datetime: 2003-02-01 00:07:36>
+#<CF Datetime: 2003-02-01T00:07:36Z>
 #
 #>>> cf.Datetime(2003, 2, 1) + cf.M(1)
-#<CF Datetime: 2003-03-01 00:00:00>
+#<CF Datetime: 2003-03-01T00:00:00Z>
 #>>> cf.M(1) + cf.Datetime(2003, 2, 1, calendar='360_day') 
-#<CF Datetime: 2003-03-01 00:00:00 360_day>
+#<CF Datetime: 2003-03-01T00:00:00Z 360_day>
 #>>> cf.Datetime(2003, 2, 1, calendar='noleap') + cf.M(245)
-#<CF Datetime: 2023-07-01 00:00:00 noleap>
+#<CF Datetime: 2023-07-01T00:00:00Z noleap>
 #>>> cf.Datetime(2003, 2, 1) - cf.Y(2)
-#<CF Datetime: 2001-02-01 00:00:00>
+#<CF Datetime: 2001-02-01T00:00:00Z>
 #>>> cf.Datetime(2003, 2, 1) - cf.Y(2) + cf.D(10)
-#<CF Datetime: 2001-02-11 00:00:00>
+#<CF Datetime: 2001-02-11T00:00:00Z>
 #
 #>>> d = cf.Datetime(2003, 2, 1)
 #>>> d += cf.s(34567689)
 #>>> d
-#<CF Datetime: 2004-03-07 02:08:09>
+#<CF Datetime: 2004-03-07T02:08:09Z>
 #
 #
 #**Attributes**
@@ -661,7 +697,7 @@ Return the current Gregorian calendar UTC date and time.
 #        if self.microsecond:
 #            args = list(args)
 #            args[-1] += self.microsecond/1e6            
-#        return '{0: >4}-{1:0>2}-{2:0>2} {3:0>2}:{4:0>2}:{5:0>2}'.format(*args)
+#        return '{0: >4}-{1:0>2}-{2:0>2}T{3:0>2}:{4:0>2}:{5:0>2}Z'.format(*args)
 #    #--- End: def
 #
 #    def __eq__(self, other):
@@ -1025,7 +1061,7 @@ minute, second and microsecond.
 
 >>> d = cf.dt(2003)
 >>> print d
-'2003-2-30 00:00:00'
+'2003-2-30T00:00:00Z'
 >>> d = cf.dt(2003, 2, 30)
 >>> d = cf.dt(2003, 2, 30, 0, 0, 0)
 >>> d = cf.dt('2003-2-30')
@@ -1164,7 +1200,9 @@ Parse an ISO 8601 date-time string into a `cf.Datetime` object.
 array_st2Datetime = numpy_vectorize(st2Datetime, otypes=[object])
 
 def rt2dt(array, units_in, units_out=None, dummy1=None):
-    '''The returned array is always independent.
+    '''Convert reference times  to date-time objects
+
+The returned array is always independent.
 
 :Parameters:
 
@@ -1184,24 +1222,45 @@ def rt2dt(array, units_in, units_out=None, dummy1=None):
         An array of `cf.Datetime` objects with the same shape as
         *array*.
 
-    '''           
-    ndim = numpy_ndim(array)
-    
+    '''
+    mask = None
+    if numpy_ma_isMA(array):
+        # num2date has issues if the mask is nomask
+        mask = array.mask
+        if mask is numpy_ma_nomask or not numpy_ma_is_masked(array):
+            array = array.view(numpy_ndarray)
+
     array = units_in._utime.num2date(array)
 
-    if not ndim:
-        array = numpy_array(array, dtype=object)
+    if mask is not None:
+        array = numpy_ma_masked_where(mask, array)
 
-    # Return the array 
-    return numpy_vectorize(
-        functools_partial(dt2Dt, calendar=units_in._calendar), #ppp
-        otypes=[object])(array)
+    ndim = numpy_ndim(array)
+
+    if mask is None:
+        # There is no missing data
+        array = numpy_array(array, dtype=object)
+        return numpy_vectorize(
+            functools_partial(dt2Dt, calendar=units_in._calendar),
+            otypes=[object])(array)
+    else:
+        # There is missing data
+        if not ndim:
+            return numpy_ma_masked_all((), dtype=object)
+        else:
+            array = numpy_array(array)
+            array = numpy_vectorize(
+                functools_partial(dt2Dt, calendar=units_in._calendar),
+                otypes=[object])(array)
+            return numpy_ma_masked_where(mask, array)
 #--- End: def
 
 def dt2Dt(x, calendar=None):
     '''Convert a datetime.datetime object to a cf.Datetime object'''
+    if not x:
+        return False
     return Datetime(*elements(x), calendar=calendar)
-#array_dt2Dt = numpy_vectorize(functools_partial(dt2Dt, c, otypes=[object])
+#--- End: def
 
 def dt2rt(array, units_in, units_out, dummy1=None):
     '''Round to the nearest millisecond. This is only necessary whilst
@@ -1280,7 +1339,6 @@ The returned array is always independent.
                                 if year:
                                     array *= 365.242198781 * 86400.0
     #--- End: if
-
     array = numpy_around(array, decimals, array)
 
     if day:
@@ -1296,6 +1354,9 @@ The returned array is always independent.
     elif year:
         array /= 365.242198781 * 86400.0
 
+    if not ndim:
+        array = numpy_asanyarray(array)
+        
     return array
 #--- End: def
 
