@@ -840,50 +840,59 @@ array([-31., -30., -29., -28., -27.])
         self._utime     = None
     #--- End: def
 
-#    def __getstate__(self):
-#        '''
-#
-#Called when pickling.
-#
-#:Parameters:
-#
-#    None
-#
-#:Returns:
-#
-#    out : dict
-#        A dictionary of the instance's attributes
-#
-#:Examples:
-#
-#>>> u = cf.Units('days since 3-4-5', calendar='gregorian')
-#>>> u.__getstate__()
-#{'calendar': 'gregorian',
-# 'units': 'days since 3-4-5'}
-#
-#'''
-#        return dict([(attr, getattr(self, attr))
-#                     for attr in ('calendar', 'units') if hasattr(self, attr)])
-#    #--- End: def        
-#
-#    def __setstate__(self, odict):
-#        '''
-#
-#Called when unpickling.
-#
-#:Parameters:
-#
-#    odict : dict
-#        The output from the instance's `__getstate__` method.
-#
-#:Returns:
-#
-#    None
-#
-#'''
-#        for attr, value in odict.iteritems():
-#            setattr(self, attr, value)
-#     #--- End: def
+    def __getstate__(self):
+        '''
+
+Called when pickling.
+
+:Parameters:
+
+    None
+
+:Returns:
+
+    out : dict
+        A dictionary of the instance's attributes
+
+:Examples:
+
+>>> u = cf.Units('days since 3-4-5', calendar='gregorian')
+>>> u.__getstate__()
+{'calendar': 'gregorian',
+ 'units': 'days since 3-4-5'}
+
+'''
+        return dict([(attr, getattr(self, attr))
+                     for attr in ('_units', '_calendar')
+                     if hasattr(self, attr)])
+    #--- End: def        
+
+    def __setstate__(self, odict):
+        '''
+
+Called when unpickling.
+
+:Parameters:
+
+    odict : dict
+        The output from the instance's `__getstate__` method.
+
+:Returns:
+
+    None
+
+'''
+        units = None
+        if '_units' in odict:
+            units = odict['_units']
+        #--- End: if
+        calendar = None
+        if '_calendar' in odict:
+            calendar = odict['_calendar']
+        #--- End: if
+        self.__init__(units=units, calendar=calendar)
+            
+     #--- End: def
 
 #    def __hash__(self):
 #        '''
